'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useRowApi = exports.useColApi = exports.layoutApi = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var flex = { display: 'flex' };
var row = flex;
var col = _extends({}, flex, { flexDirection: 'column' });

var layoutApi = exports.layoutApi = {
  row: row,
  col: col,
  rowx: { justifyContent: 'center' },
  rowy: { alignItems: 'center' },
  colx: { alignItems: 'center' },
  coly: { justifyContent: 'center' },
  rowxEnd: {
    justifyContent: 'flex-end'
  },
  colxEnd: {
    alignItems: 'flex-end'
  },
  rowyEnd: {
    alignItems: 'flex-end'
  },
  colyEnd: {
    justifyContent: 'flex-end'
  },
  reverse: {
    flexDirection: 'row-reverse'
  },
  columnReverse: {
    flexDirection: 'column-reverse'
  },
  wrap: {
    flexWrap: 'wrap'
  },
  wrapReverse: {
    flexWrap: 'wrap-reverse'
  },
  spaceBetween: {
    justifyContent: 'space-between'
  },
  spaceAround: {
    justifyContent: 'space-around'
  }
};

var axisConfig = {
  row: {
    x: {
      true: 'rowx',
      end: 'rowxEnd'
    },
    y: {
      true: 'rowy',
      end: 'rowyEnd'
    }
  },
  col: {
    x: {
      true: 'colx',
      end: 'colxEnd'
    },
    y: {
      true: 'coly',
      end: 'colyEnd'
    }
  }
};

var useColApi = exports.useColApi = useLayoutApi('col');
var useRowApi = exports.useRowApi = useLayoutApi('row');

/**
 * 1. When applying alignItems or JustifyContent properties we will give
 * preference to the spaceProp.
 */
function useLayoutApi(type) {
  var axisPropConfig = axisConfig[type];
  return function (Style) {
    var LayoutApi = function LayoutApi(_ref) {
      var _extends2;

      var x = _ref.x;
      var y = _ref.y;
      var space = _ref.space;
      var wrap = _ref.wrap;
      var reverse = _ref.reverse;

      var props = _objectWithoutProperties(_ref, ['x', 'y', 'space', 'wrap', 'reverse']);

      var xAxisProp = axisPropConfig.x[x];
      var yAxisProp = axisPropConfig.y[y];
      var spaceProp = false;
      var wrapProp = false;
      var reverseProp = false;

      if (space) {
        spaceProp = 'space' + (0, _utils.capitalize)(space);
      }

      if (wrap === true) {
        wrapProp = 'wrap';
      } else if (wrap === 'reverse') {
        wrapProp = 'wrapReverse';
      }

      if (reverse) {
        reverseProp = type === 'row' ? 'row-reverse' : 'columne-reverse';
      }

      return _react2.default.createElement(Style, _extends({}, props, (_extends2 = {}, _defineProperty(_extends2, type, true), _defineProperty(_extends2, xAxisProp, true), _defineProperty(_extends2, yAxisProp, true), _defineProperty(_extends2, spaceProp, true), _defineProperty(_extends2, wrapProp, true), _defineProperty(_extends2, reverseProp, true), _extends2)));
    };

    return LayoutApi;
  };
}